// -*- C++ -*-
/*!
 * @file  NM33CameraCapture.h
 * @brief 接続した８台のNM33カメラからカメラ画像の取得を行う
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#ifndef NM33CAMERACAPTURE_H
#define NM33CAMERACAPTURE_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
//add
#include "local_basic_type.h"
#include "NM33BirdsEyeSystem.h"
#include "NM33CameraSSKFunc.h"

//Opencv関連のインクルード宣言
#include <opencv2/opencv.hpp>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "NM33ControlSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

/*====================================================
	マクロ定義
 ===================================================*/
//マトリクス用
#define CAPTURE_NOACTION	(0)		//何もしない
#define CAPTURE_ONCE		(1)		//一回キャプチャする
#define CAPTURE_CYCLE		(2)		//周期的にキャプチャする
#define CAPTURE_STOP		(3)		//周期的なキャプチャを停止する

//メッセージ用
#define MES_CAMERA_CONNECT_SUCCESS		"[MESSEGE]Camera Connect Success!!"								//カメラ接続に成功しました
#define MES_CAMERA_CONNECT_NUM			"[MESSEGE]Camera Connect Num = "								//カメラ接続台数はXX
#define MES_CAMERA_CONNECT_ERROR		"[ERROR MESSEGE]Camera Connect Error!!"							//カメラ接続に失敗しました
#define MES_CAMERA_NO_CONNECT			"[ERROR MESSEGE]Camera No Connect!!"							//カメラが接続されていません
#define MES_CAMERA_CONNECT_RANGE_OUT	"[ERROR MESSEGE]Camera No Connect Or Connection Out Of Range!!"	//カメラ未接続又は範囲外の台数を接続しています
#define MES_CAMERA_EXEMPT_ERROR			"[ERROR MESSEGE]Support Outside Camera Connect!!"				//サポート外のカメラが接続されています
#define MES_CAMERA_CAPTURE_ERROR		"[ERROR MESSEGE]Camera Capture Error!!"							//画像のキャプチャに失敗しました
#define MES_CAMERA_CAPTURE_DAMAGE		"[ERROR MESSEGE]Camera Capture Damage!!"						//キャプチャした画像が破損しています
#define MES_CAMERA_OPEN_ERROR			"[ERROR MESSEGE]Camera Open Error = "							//カメラオープンに失敗しました
#define MES_SERIALNO_GET_ERROR			"[ERROR MESSEGE]Serial No Get Error!!"							//シリアルNoの取得に失敗しました
#define MES_CAMERA_CAP_CREATE_ERROR		"[ERROR MESSEGE]Camera Capture File Create Error!!"				//カメラキャプチャ画像の生成に失敗しました
#define MES_FOLDER_CREATE_ERROR			"[ERROR MESSEGE]Camera Capture Folder Create Error!!"			//フォルダの生成に失敗しました
#define MES_RECOVERY_RESET_RETRY_CNT	"[MESSEGE]Recovery Reset Retry Count = "						//リカバリリセットのRetry回数はXX

/*!
 * @class NM33CameraCapture
 * @brief 接続した８台のNM33カメラからカメラ画像の取得を行う
 *
 * 別紙"機能仕様書_カメラ画像取得RTC"参照
 *
 * 別紙"機能仕様書_カメラ画像取得RTC"参照
 *
 * 別紙"機能仕様書_カメラ画像取得RTC"参照
 *
 * 別紙"機能仕様書_カメラ画像取得RTC"参照
 *
 */
class NM33CameraCapture
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  NM33CameraCapture(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~NM33CameraCapture();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * カメラ画像取得RTCの初期化処理
   * 　モード状態などの初期化を行う
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * カメラ画像取得RTCの終了処理
   * 　NM33用ドライバの終了処理（disconnect）を行う
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post onActivatedするまで、NM33用ドライバは使用不可
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * カメラ画像取得RTCの開始処理
   * 　NM33用ドライバなどの初期化を行う
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * カメラ画像取得RTCの終了処理
   * 　NM33用ドライバの終了処理（disconnect）を行う
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post onActivatedするまで、NM33用ドライバは使用不可
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * カメラキャプチャを行う周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre カメラを一台以上接続していること
   * ※カメラ未接続状態の場合エラーを検出しエラー状態へ遷移す
   * る
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * カメラ画像取得RTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post システムリセットすることでエラー状態から復帰する
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   * カメラ画像取得RTCのエラー処理
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post システムリセットすることでエラー状態から復帰する（Reset
   * へ遷移）
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   * カメラ画像取得RTCのリセット処理
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post システムリセットすることでReset状態から復帰する（Activa
   * teへ遷移）
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_A SerialNo_A
   * - DefaultValue: NM33-036233
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_A;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_B SerialNo_B
   * - DefaultValue: NM33-036235
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_B;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_C SerialNo_C
   * - DefaultValue: NM33-036236
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_C;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_D SerialNo_D
   * - DefaultValue: NM33-036240
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_D;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_E SerialNo_E
   * - DefaultValue: NM33-036279
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_E;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_F SerialNo_F
   * - DefaultValue: NM33-036280
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_F;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_G SerialNo_G
   * - DefaultValue: NM33-036284
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_G;
  /*!
   * 接続カメラを識別するために使用する
   * - Name: SerialNo_H SerialNo_H
   * - DefaultValue: NM33-036286
   * - Unit: なし
   * - Range: なし
   * - Constraint: なし
   */
  std::string m_SerialNo_H;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::MultiCameraImages m_MultiCaptureimage;
  /*!
   * カメラID =
   * 0～7の内、接続されているカメラのキャプチャ画像を出力する
   * キャプチャ画像はコンフィギュレーションで設定したシリアルNo順に
   * 格納される
   * - Type: RTC::MultiCameraImages
   * - Number: 8
   * - Semantics: -
   * - Unit: -
   * - Frequency: 1秒周期
   * - Operation Cycle: 1秒周期
   */
  OutPort<RTC::MultiCameraImages> m_MultiCaptureimageOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * カメラ画像取得動作を制御する
   * Interface: 変換モードや撮影モードの設定及び、キャプチャ動作要
   *            求などのI/Fを提供する
   */
  RTC::CorbaPort m_NM33CameraControlServPort;
  /*!
   * カメラ画像取得RTCのシステム制御
   * Interface: リセット制御のI/Fを提供する
   */
  RTC::CorbaPort m_NM33CameraSystemContServPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * カメラ画像をキャプチャする際の変換モードの設定及び、状態を取得
   * する
   * 　①SetNM33ChangeMode　・・・　設定処理
   * 　②GetNM33ChangeMode　・・・　取得処理
   * - Argument:      ①SetNM33ChangeModeの引数　・・・　
   *                  　・CHG_MODE_DEFAULT：変換なし
   *                  　・CHG_MODE_PERSPECTIVE：透視投影
   *                  　・CHG_MODE_BIRDS_EYE：俯瞰合成
   *                  ②GetNM33ChangeModeの引数　・・・　なし
   * - Return Value:  ①SetNM33ChangeModeの戻り値　・・・　なし
   *                  ②GetNM33ChangeModeの戻り値　・・・　
   *                  　・CHG_MODE_DEFAULT：変換なし
   *                  　・CHG_MODE_PERSPECTIVE：透視投影
   *                  　・CHG_MODE_BIRDS_EYE：俯瞰合成
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  NM33ChangeModeSVC_impl m_NM33ChangeModeProv;
  /*!
   * カメラ画像をキャプチャする際の撮影モードの設定及び、状態を取得
   * する
   * 　①SetNM33CaptureMode　・・・　設定処理
   * 　②GetNM33CaptureMode　・・・　取得処理
   * - Argument:      ①SetNM33CaptureModeの引数　・・・　
   *                  　・CAPT_MODE_PICTURE：静止画
   *                  　・CAPT_MODE_VIDEO：動画
   *                  ②GetNM33CaptureModeの引数　・・・　なし
   * - Return Value:  ①SetNM33CaptureModeの戻り値　・・・　なし
   *                  ②GetNM33CaptureModeの戻り値　・・・　
   *                  　・CAPT_MODE_PICTURE：静止画
   *                  　・CAPT_MODE_VIDEO：動画
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  NM33CaptureModeSVC_impl m_NM33CaptureModeProv;
  /*!
   * キャプチャ動作の開始/停止の要求及び、状態を取得する
   * 　①ReqNM33CaptureTrigger　・・・　要求処理
   * 　②GeqNM33CaptureTrigger　・・・　取得処理
   * - Argument:      ①ReqNM33CaptureTriggerの引数　・・・　
   *                  　・CAPT_TRIG_START：開始
   *                  　・CAPT_TRIG_STOP：停止
   *                  ②GeqNM33CaptureTriggerの引数　・・・　なし
   * - Return Value:  ①ReqNM33CaptureTriggerの戻り値　・・・　なし
   *                  ②GeqNM33CaptureTriggerの戻り値　・・・　
   *                  　・CAPT_TRIG_START：開始
   *                  　・CAPT_TRIG_STOP：停止
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  NM33CaptureTriggerSVC_impl m_NM33CaptureTriggerProv;
  /*!
   * カメラ画像取得RTCのリセットの要求及び、状態を取得する
   * 　①ReqNM33CaptureReset　・・・　要求処理
   * 　②GeqNM33CaptureReset　・・・　取得処理
   * - Argument:      ①ReqNM33CaptureResetの引数　・・・　
   *                  　・RESET_TRIG_ON：Reset On
   *                  　・RESET_TRIG_OFF：Reset Off
   *                  ②GeqNM33CaptureResetの引数　・・・　なし
   * - Return Value:  ①ReqNM33CaptureResetの戻り値　・・・　なし
   *                  ②GeqNM33CaptureResetの戻り値　・・・　
   *                  　・RESET_TRIG_ON：Reset On
   *                  　・RESET_TRIG_OFF：Reset Off
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  NM33CameraResetSVC_impl m_NM33CameraResetProv;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  //add
  uWord_Type CameraCaptureActMatrix(uWord_Type *);					//マトリクスチェック処理
  uWord_Type CameraChageModeSet(void);								//接続カメラの変換モード切替処理
  uWord_Type MultiCameraCapture(uWord_Type);						//複数のカメラキャプチャ処理
  uWord_Type MultiCameraSerialSort(uWord_Type, uWord_Type *);		//シリアルNoソート処理
  uWord_Type CameraCaptureByOpcv(uWord_Type, uWord_Type);			//Opencvによるカメラキャプチャ処理
  uWord_Type MultiCameraOpen(void);									//カメラオープン処理
  uWord_Type MultiCameraClose(void);								//カメラクローズ処理
  void CameraCapFileName(void);										//キャプチャ画像ファイル名作成処理
  uWord_Type CameraCapJPGFileCreate(uWord_Type);					//キャプチャ画像JPGファイル保存処理
  void NM33CameraCapture::CameraCapFPSLogOut(void);					//キャプチャ周期のFPSログ出力

  uWord_Type m_uWBackUpCangeMode;			//変換モードのバックアップ(変化チェック用)
  uWord_Type m_uWBackUpCaptureMode;			//撮影モードのバックアップ(変化チェック用)
  uWord_Type m_uWBackUpCaptureTrigger;		//キャプチャ動作トリガのバックアップ(変化チェック用)
  uWord_Type m_uWBackUpCameraNum;			//起動時点のカメラ接続台数(変化チェック用)

  struct tm	m_stBackUpFileCreateTime;		//画像ファイル生成時の時間(変化チェック用)
  uWord_Type m_uWFileBranchNum;				//ファイル名の枝番
  uWord_Type m_uWResetCnt;					//リセット時のRetryカウント
  uWord_Type m_uWReqReset;					//リセット要求を保持
  Bool_Type m_blErrSts;						//エラー状態が確定したかを保持
};

//add
//Error状態からActivateへ遷移させる為のクラス
class activator
{
  RTC::UniqueId ec_id;
public:
  activator(RTC::UniqueId _ec_id) : ec_id(_ec_id) {}
  void operator()(RTObject_impl* comp)
  {
    Sleep(100);
    comp->activate(ec_id);
  }
};


extern "C"
{
  DLL_EXPORT void NM33CameraCaptureInit(RTC::Manager* manager);
};

#endif // NM33CAMERACAPTURE_H
