/*
 * File Name		: NM33CameraSSKFunc.h
 * Function			: NM33用のライブラリを使用しカメラ制御を行う
 * Author			: takashi sato
 * Date				: 2015/11/26
 * Last-modified by	: 
 * Last-modified	: 
 * History log		: 
 */

#ifndef NM33CAMERASSKFUNC_H
#define NM33CAMERASSKFUNC_H

#include "NM33CameraCapture.h"
#include "local_basic_type.h"

//OptCamSDK制御用
#include <windows.h>
#include <SSK.h>

/*==============================================================================
	マクロ定義
==============================================================================*/

//NM33 SSKの関数実行結果用
enum SSK_RESULT{
	SSK_RESULT_SUCCESS = 0,				//正常終了
	SSK_RESULT_ERR,						//異常終了
	SSK_RESULT_CONNECT_ERR,				//異常終了（接続エラー）
	SSK_RESULT_INIT_ERR,				//異常終了（初期化エラー）
};

//NM33カメラ接続数
enum NM33_CONNECT_NUM{
	NM33_CONNECT_0 = 0,					//0台（未使用）
	NM33_CONNECT_1,						//1台
	NM33_CONNECT_2,						//2台
	NM33_CONNECT_3,						//3台
	NM33_CONNECT_4,						//4台
	NM33_CONNECT_5,						//5台
	NM33_CONNECT_6,						//6台
	NM33_CONNECT_7,						//7台
	NM33_CONNECT_8,						//8台
	NM33_CONNECT_OVER,					//8台以上
};

//画面モード
enum PANORAMA_MODE{
	PANORAMA_MODE_CIRCLE,				//円形
	PANORAMA_MODE_AROUND,				//周回
	PANORAMA_MODE_AROUND_TURN,			//周回（反転）
	PANORAMA_MODE_HALF,					//上下二分割
	PANORAMA_MODE_HALF_TURN,			//上下二分割（反転）
	PANORAMA_MODE_FOUR_SCREEN,			//4画面
	PANORAMA_MODE_FOUR_SCREEN_TURN,		//4画面（反転）
	PANORAMA_MODE_DUMMY_1,				//未使用
	PANORAMA_MODE_DUMMY_2,				//未使用
	PANORAMA_MODE_CIRCLE_LARGE,			//円形（Largeサイズ）
	PANORAMA_MODE_DUMMY_3,				//未使用
	PANORAMA_MODE_PERSPECTIVE,			//直交一画面
	PANORAMA_MODE_MAX,
};

#define SSK_ERROR_RETRY_MAX		(5)		//エラーリトライMax回数

/*=============================================================================
	クラス宣言
==============================================================================*/
class NM33CameraSSKFunc{
public:
	NM33CameraSSKFunc();
	~NM33CameraSSKFunc();

	uWord_Type	NM33CameraInit(void);												//カメラ初期化処理
	void		NM33CameraDisconnect(void);											//カメラ切断処理
	uWord_Type	NM33CameraGetNumber(void);											//カメラ接続台数取得処理
	Char_Type*	NM33CameraGetSerialNo(uWord_Type);									//シリアルNo取得処理
	uWord_Type	NM33CameraSetPanoramaMode(uWord_Type, uWord_Type);					//画面モード設定処理
//protected:

private:
	uWord_Type NM33CameraInitWrap(void);											//カメラ接続初期化処理
	uWord_Type	NM33CameraDisconnectWrap(void);										//カメラ接続切断処理
	uWord_Type	NM33CameraGetNumbertWrap(uWord_Type *);								//接続カメラ数取得処理
	uWord_Type	NM33CameraSetPanoramaModeWrap(uWord_Type, uWord_Type);				//画面モード設定処理
	uWord_Type	NM33CameraGetPanoramaModeWrap(uWord_Type, uWord_Type *);			//画面モード取得処理
	Char_Type*	NM33CameraGetSerialNoWrap(uWord_Type);								//シリアルNo取得処理
};

#endif // NM33CAMERASSKFUNC_H



