// -*- C++ -*-
/*!
 * @file  NM33CameraCapture.cpp
 * @brief 接続した８台のNM33カメラからカメラ画像の取得を行う
 * @date $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * $Id$
 */

#include "NM33CameraCapture.h"
#include <coil/Async.h>

// Module specification
// <rtc-template block="module_spec">
static const char* nm33cameracapture_spec[] =
  {
    "implementation_id", "NM33CameraCapture",
    "type_name",         "NM33CameraCapture",
    "description",       "接続した８台のNM33カメラからカメラ画像の取得を行う",
    "version",           "0.1",
    "vendor",            "EAC",
    "category",          "カメラ",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.SerialNo_A", "NM33-036233",
    "conf.default.SerialNo_B", "NM33-036235",
    "conf.default.SerialNo_C", "NM33-036236",
    "conf.default.SerialNo_D", "NM33-036240",
    "conf.default.SerialNo_E", "NM33-036279",
    "conf.default.SerialNo_F", "NM33-036280",
    "conf.default.SerialNo_G", "NM33-036284",
    "conf.default.SerialNo_H", "NM33-036286",
    // Widget
    "conf.__widget__.SerialNo_A", "text",
    "conf.__widget__.SerialNo_B", "text",
    "conf.__widget__.SerialNo_C", "text",
    "conf.__widget__.SerialNo_D", "text",
    "conf.__widget__.SerialNo_E", "text",
    "conf.__widget__.SerialNo_F", "text",
    "conf.__widget__.SerialNo_G", "text",
    "conf.__widget__.SerialNo_H", "text",
    // Constraints
    ""
  };
// </rtc-template>

//add
NM33CameraSSKFunc	*m_NM33CameraSSKFunc_p = NULL;
cv::VideoCapture	m_CVCap_1;
cv::VideoCapture	m_CVCap_2;
cv::VideoCapture	m_CVCap_3;
cv::VideoCapture	m_CVCap_4;
cv::VideoCapture	m_CVCap_5;
cv::VideoCapture	m_CVCap_6;
cv::VideoCapture	m_CVCap_7;
cv::VideoCapture	m_CVCap_8;

//VideoCaptuerオープン用テーブル
cv::VideoCapture *VideoCapOpenTbl[NM33_CONNECT_ENABLE_MAX] = {
	&m_CVCap_1,
	&m_CVCap_2,
	&m_CVCap_3,
	&m_CVCap_4,
	&m_CVCap_5,
	&m_CVCap_6,
	&m_CVCap_7,
	&m_CVCap_8
};


/*!
 * @brief constructor
 * @param manager Maneger Object
 */
NM33CameraCapture::NM33CameraCapture(RTC::Manager* manager)
	// <rtc-template block="initializer">
	: RTC::DataFlowComponentBase(manager),
	m_MultiCaptureimageOut("MultiCaptureDataOut", m_MultiCaptureimage),
	m_NM33CameraControlServPort("NM33CameraControlServ"),
	m_NM33CameraSystemContServPort("NM33CameraSystemContServ"),

	//add
	m_uWBackUpCangeMode(CHG_MODE_DEFAULT),
	m_uWBackUpCaptureMode(CAPT_MODE_VIDEO),
	m_uWBackUpCaptureTrigger(CAPT_TRIG_STOP),
	m_uWBackUpCameraNum(NM33_CONNECT_0),
	m_uWFileBranchNum(0),
	m_uWResetCnt(1),
	m_uWReqReset(RESET_TRIG_OFF),
	m_blErrSts(FALSE)

	// </rtc-template>
{
	::memset(&m_stBackUpFileCreateTime, 0, sizeof(tm));
}

/*!
 * @brief destructor
 */
NM33CameraCapture::~NM33CameraCapture()
{
}


/*!
 * カメラ画像取得RTCの初期化処理
 * 　モード状態などの初期化を行う
 */
RTC::ReturnCode_t NM33CameraCapture::onInitialize()
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onInitialize()" << std::endl;
#endif

	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers

	// Set OutPort buffer
	addOutPort("MultiCaptureDataOut", m_MultiCaptureimageOut);

	// Set service provider to Ports
	m_NM33CameraControlServPort.registerProvider("NM33ChangeMode", "NM33CameraControlServ::NM33ChangeMode", m_NM33ChangeModeProv);
	m_NM33CameraControlServPort.registerProvider("NM33CaptureMode", "NM33CameraControlServ::NM33CaptureMode", m_NM33CaptureModeProv);
	m_NM33CameraControlServPort.registerProvider("NM33CaptureTrigger", "NM33CameraControlServ::NM33CaptureTrigger", m_NM33CaptureTriggerProv);
	m_NM33CameraSystemContServPort.registerProvider("NM33CameraReset", "NM33CameraSystemContServ::NM33CameraReset", m_NM33CameraResetProv);

	// Set service consumers to Ports

	// Set CORBA Service Ports
	addPort(m_NM33CameraControlServPort);
	addPort(m_NM33CameraSystemContServPort);

	// </rtc-template>

	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("SerialNo_A", m_SerialNo_A, "NM33-036233");
	bindParameter("SerialNo_B", m_SerialNo_B, "NM33-036235");
	bindParameter("SerialNo_C", m_SerialNo_C, "NM33-036236");
	bindParameter("SerialNo_D", m_SerialNo_D, "NM33-036240");
	bindParameter("SerialNo_E", m_SerialNo_E, "NM33-036279");
	bindParameter("SerialNo_F", m_SerialNo_F, "NM33-036280");
	bindParameter("SerialNo_G", m_SerialNo_G, "NM33-036284");
	bindParameter("SerialNo_H", m_SerialNo_H, "NM33-036286");
	// </rtc-template>

	//add
	m_NM33CameraSSKFunc_p = new NM33CameraSSKFunc();

	return RTC::RTC_OK;
}

/*!
 * カメラ画像取得RTCの終了処理
 * 　NM33用ドライバの終了処理（disconnect）を行う
 */

RTC::ReturnCode_t NM33CameraCapture::onFinalize()
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onFinalize()" << std::endl;
#endif

	if(m_NM33CameraSSKFunc_p == NULL){
		return RTC_ERROR;
	}
	//NM33ドライバ終了処理
	m_NM33CameraSSKFunc_p->NM33CameraDisconnect();

	if(m_NM33CameraSSKFunc_p != NULL){
		delete m_NM33CameraSSKFunc_p;
	}

	//ビデオキャプチャを解放
	MultiCameraClose();

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t NM33CameraCapture::onStartup(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t NM33CameraCapture::onShutdown(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}
*/

/*!
 * カメラ画像取得RTCの開始処理
 * 　NM33用ドライバなどの初期化を行う
 */

RTC::ReturnCode_t NM33CameraCapture::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onActivated()" << std::endl;
#endif

//↓↓add
	uWord_Type uWSSKFunkResult = SSK_RESULT_SUCCESS;
	uWord_Type uWResult = RESULT_OK;
	uWord_Type uWIndex = 0;

	if(m_NM33CameraSSKFunc_p == NULL){
		return RTC::RTC_ERROR;
	}

	//NM33ドライバ終了処理
	uWSSKFunkResult = m_NM33CameraSSKFunc_p->NM33CameraInit();
	switch(uWSSKFunkResult){
		case SSK_RESULT_SUCCESS:
			m_uWBackUpCameraNum = m_NM33CameraSSKFunc_p->NM33CameraGetNumber();
			if((m_uWBackUpCameraNum == NM33_CONNECT_0) || (m_uWBackUpCameraNum > NM33_CONNECT_ENABLE_MAX)){
				std::cout << MES_CAMERA_CONNECT_RANGE_OUT << std::endl;
				return RTC::RTC_ERROR;
			}else{
				std::cout << MES_CAMERA_CONNECT_SUCCESS << std::endl;
				std::cout << MES_CAMERA_CONNECT_NUM << m_uWBackUpCameraNum << std::endl;
			}

			//カメラオープン
			uWResult = MultiCameraOpen();
			if(uWResult == RESULT_NG){
				return RTC::RTC_ERROR;
			}

			//接続最大数分の領域を確保
			m_MultiCaptureimage.images.length(NM33_CONNECT_ENABLE_MAX);
			for(uWIndex = 0; uWIndex < NM33_CONNECT_ENABLE_MAX; uWIndex++){
				m_MultiCaptureimage.images[uWIndex].width = 0;
				m_MultiCaptureimage.images[uWIndex].height = 0;
			}
		break;

		case SSK_RESULT_CONNECT_ERR:
			std::cout << MES_CAMERA_NO_CONNECT << std::endl;
			return RTC::RTC_ERROR;

		case SSK_RESULT_INIT_ERR:
		default:
			std::cout << MES_CAMERA_CONNECT_ERROR << std::endl;
			return RTC::RTC_ERROR;
	}
//↑↑end

	return RTC::RTC_OK;
}

/*!
 * カメラ画像取得RTCの終了処理
 * 　NM33用ドライバの終了処理（disconnect）を行う
 */

RTC::ReturnCode_t NM33CameraCapture::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onDeactivated()" << std::endl;
#endif

	//↓↓add 
	if(m_NM33CameraSSKFunc_p == NULL){
		return RTC_ERROR;
	}
	//NM33ドライバ終了処理
	m_NM33CameraSSKFunc_p->NM33CameraDisconnect();

	//ビデオキャプチャを解放
	MultiCameraClose();

	m_uWBackUpCangeMode = CHG_MODE_DEFAULT;
	m_uWBackUpCaptureMode = CAPT_MODE_VIDEO;
	m_uWBackUpCaptureTrigger = CAPT_TRIG_STOP;
	m_uWBackUpCameraNum = NM33_CONNECT_0;
	m_uWFileBranchNum = 0;
	::memset(&m_stBackUpFileCreateTime, 0, sizeof(tm));
//↑↑end

	return RTC::RTC_OK;
}

/*!
 * カメラキャプチャを行う周期処理
 */

RTC::ReturnCode_t NM33CameraCapture::onExecute(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onExecute()" << std::endl;
#endif

//↓↓add 
	uWord_Type uWResult = RESULT_OK;
	uWord_Type uWMatrixMode = CAPTURE_NOACTION;

	//マトリクスチェック
	uWResult = CameraCaptureActMatrix(&uWMatrixMode);

	if(uWResult == RESULT_NG){
		return RTC::RTC_ERROR;
	}

	//キャプチャ
	if((uWMatrixMode == CAPTURE_ONCE) || (uWMatrixMode == CAPTURE_CYCLE)){
		uWResult = 	MultiCameraCapture(uWMatrixMode);

		//キャプチャモードは一回のみの動作の為、動作完了後トリガをOFFする
		if(uWMatrixMode == CAPTURE_ONCE){
			m_NM33CaptureTriggerProv.ReqNM33CaptureTrigger(CAPT_TRIG_STOP);
			m_uWBackUpCaptureTrigger = CAPT_TRIG_STOP;
		}

		//キャプチャ処理の実行結果を判定
		if(uWResult == RESULT_NG){
			std::cout << MES_CAMERA_CAPTURE_ERROR << std::endl;
			return RTC::RTC_ERROR;
		}

		//データポートの書き込み
		m_MultiCaptureimageOut.write();	

		//FPSのログ出力
		CameraCapFPSLogOut();
	}

	//ResetによるRetryカウントをクリアする
	m_uWResetCnt = 1;

//↑↑end
	return RTC::RTC_OK;
}

/*!
 * カメラ画像取得RTCのエラー遷移処理
 */

RTC::ReturnCode_t NM33CameraCapture::onAborting(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onAborting()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*!
 * カメラ画像取得RTCのエラー処理
 */

RTC::ReturnCode_t NM33CameraCapture::onError(RTC::UniqueId ec_id)
{
	m_uWReqReset = m_NM33CameraResetProv.GetNM33CaptureReset();

	if((m_uWReqReset == RESET_TRIG_ON) && (m_uWResetCnt <= RESET_RETRY_MAX_CNT)){
		//RTCの状態をResetへ遷移させる
		get_context(ec_id)->reset_component(getObjRef());
		std::cout << MES_RECOVERY_RESET_RETRY_CNT << m_uWResetCnt << std::endl;
	}else{
		//エラー状態が確定している
		m_blErrSts = TRUE;
	}

	return RTC::RTC_OK;
}

/*!
 * カメラ画像取得RTCのリセット処理
 */

RTC::ReturnCode_t NM33CameraCapture::onReset(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraCapture::onReset()" << std::endl;
#endif

	//リカバリResetのRetry処理
	if((m_uWReqReset == RESET_TRIG_ON) && (m_uWResetCnt <= RESET_RETRY_MAX_CNT)){
		m_uWResetCnt++;
		// 非同期処理にてRTCの状態をactivateへ遷移させる
		coil::AsyncInvoker(this, activator(ec_id), true)->invoke();
	}

	//RT System Editor状態でReset操作した時に動作する。既にエラー状態が確定している場合カウントをクリアする
	if(m_blErrSts == TRUE){
		//リカバリReset Retry回数を初期化
		m_uWResetCnt = 1;

		//エラー状態を初期化
		m_blErrSts = FALSE;
	}

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t NM33CameraCapture::onStateUpdate(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t NM33CameraCapture::onRateChanged(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}
*/

/*******************************************************************************
	サブルーチン
********************************************************************************/
/*******************************************************************************
 *	Function Name		：CameraCaptureActMatrix
 *	Description			：動作マトリクスの確認を行い動作を決定する
 *	Argument			：Out：マトリクスモード
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::CameraCaptureActMatrix(uWord_Type *uWMatrixMode)
{
	//マトリクステーブル
	uWord_Type CameraActMaxtixTbl[2][2]={
			//キャプチャ		//ムービー
		{	CAPTURE_NOACTION,	CAPTURE_STOP	},	//停止
		{	CAPTURE_ONCE	,	CAPTURE_CYCLE	},	//開始
	};

	//サービスポートから最新の変換モード/撮影モード/動作トリガを取得
	uWord_Type uWCurrentChgMode = m_NM33ChangeModeProv.GetNM33ChangeMode();
	uWord_Type uWCurrentCapMode = m_NM33CaptureModeProv.GetNM33CaptureMode();
	uWord_Type uWCurrentTrigMode = m_NM33CaptureTriggerProv.GetNM33CaptureTrigger();

	Bool_Type blMatrixCheck = FALSE;
	uWord_Type uWReturn = RESULT_OK;

	if(uWMatrixMode == NULL){
		return (RESULT_NG);
	}else{
		*uWMatrixMode = CAPTURE_NOACTION;
	}

	//範囲チェック
	if((uWCurrentChgMode >= CHG_MODE_MAX) || (uWCurrentCapMode >= CAPT_MODE_MAX) || (uWCurrentTrigMode >= CAPT_TRIG_MAX)){
		return (RESULT_NG);
	}

	//変化チェック
	if(m_uWBackUpCaptureMode != uWCurrentCapMode){
		m_uWBackUpCaptureMode = uWCurrentCapMode;
	}

	if(m_uWBackUpCaptureMode == CAPT_MODE_VIDEO){
		//ムービーモードの時のみ変換モードを変更する（NM33ドライバ側の変換モードの切り替えに時間がかかる為）
		if(m_uWBackUpCangeMode != uWCurrentChgMode){
			m_uWBackUpCangeMode = uWCurrentChgMode;

			//接続カメラの変換モードを切り替える
			uWReturn = CameraChageModeSet();
			if(uWReturn == RESULT_NG){
				return (RESULT_NG);
			}
		}
	}

	if(m_uWBackUpCaptureTrigger != uWCurrentTrigMode){
		m_uWBackUpCaptureTrigger = uWCurrentTrigMode;
		//動作トリガに変化がある場合、無条件でマトリクスの確認を行う
		blMatrixCheck = TRUE;
	}else{
		if(m_uWBackUpCaptureMode == CAPT_MODE_VIDEO){
			//動画モードはサイクルで動作する為、常にマトリクスの確認を行う
			blMatrixCheck = TRUE;
		}else{
			//静止画モードは、1回のみ動作の為動作トリガが変化した時のみマトリクスの確認を行う
		}
	}

	//動作トリガに変化があればマトリクスの確認を行う
	if(blMatrixCheck == TRUE){
		*uWMatrixMode = CameraActMaxtixTbl[uWCurrentTrigMode][uWCurrentCapMode];
	}

	return (RESULT_OK);
}

/*******************************************************************************
 *	Function Name		：CameraChageModeSet
 *	Description			：変換モードに従って接続カメラの画面モードを切り替える処理
 *	Argument			：なし
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::CameraChageModeSet(void)
{
	uWord_Type uWSSKFunkResult = SSK_RESULT_SUCCESS;
	uWord_Type uWCameraID = CAMERA_ID_0;

	if(m_NM33CameraSSKFunc_p == NULL){
		return (RESULT_NG);
	}

	if(m_uWBackUpCameraNum > NM33_CONNECT_ENABLE_MAX){
		return (RESULT_NG);
	}

	for(uWCameraID = 0; uWCameraID < m_uWBackUpCameraNum; uWCameraID++){
		//NM33カメラの画面モードの設定
		uWSSKFunkResult = m_NM33CameraSSKFunc_p->NM33CameraSetPanoramaMode((uWCameraID + 1), m_uWBackUpCangeMode);
		if(uWSSKFunkResult == SSK_RESULT_ERR){
			return (RESULT_NG);
		}
	}

	return (RESULT_OK);
}

/*******************************************************************************
 *	Function Name		：MultiCameraCapture
 *	Description			：8台のカメラ画像を取得する処理
 *	Argument			：In：マトリクスモード
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::MultiCameraCapture(uWord_Type uWMatrixMode)
{
	uWord_Type uWCameraID = CAMERA_ID_0;
	uWord_Type uWResult = RESULT_OK;
	uWord_Type uWIndex = 0;

	if(uWMatrixMode == CAPTURE_ONCE){
		//キャプチャ画像をJPGファイルに保存する時のファイル名を作成する
		CameraCapFileName();
	}

	//キャプチャデータ取得前に格納領域を初期化
	for(uWIndex = 0; uWIndex < NM33_CONNECT_ENABLE_MAX; uWIndex++){
		m_MultiCaptureimage.images[uWIndex].width = 0;
		m_MultiCaptureimage.images[uWIndex].height = 0;
	}

	//接続しているカメラ台数分キャプチャデータを取得する
	for(uWCameraID = 0; uWCameraID < m_uWBackUpCameraNum; uWCameraID++){
		//OpenCVによるキャプチャデータの取得
		uWResult = CameraCaptureByOpcv(uWCameraID, uWMatrixMode);
		if(uWResult == RESULT_NG){
			return (RESULT_NG);
		}
	}

	return (RESULT_OK);
}

/*******************************************************************************
 *	Function Name		：MultiCameraCapture
 *	Description			：OpenCVを使用したカメラキャプチャ処理
 *	Argument			：In：カメラID/マトリクスモード
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::CameraCaptureByOpcv(uWord_Type uWCameraID, uWord_Type uWMatrixMode)
{
	//ビデオキャプチャの取得テーブル
	cv::VideoCapture VideoCapGetTbl[NM33_CONNECT_ENABLE_MAX] = {
		m_CVCap_1,
		m_CVCap_2,
		m_CVCap_3,
		m_CVCap_4,
		m_CVCap_5,
		m_CVCap_6,
		m_CVCap_7,
		m_CVCap_8
	};

	cv::Mat Frame;
	uWord_Type uWIndex = 0;
	uWord_Type uWResult = RESULT_OK;
	uHWord_Type uHWErrorLine = 0;

	if((uWCameraID > CAMERA_ID_MAX)){
		std::cout << MES_CAMERA_CONNECT_RANGE_OUT << std::endl;
		goto CAMERA_CAPTURE_ERROR;
	}

	//引数で指定されたカメラIDから対象のキャプチャ画像を入手
	VideoCapGetTbl[uWCameraID] >> Frame;

	//SerialNo順でソート
	uWResult = MultiCameraSerialSort(uWCameraID, &uWIndex);
	if(uWResult == RESULT_NG){
		uHWErrorLine = __LINE__;
		goto CAMERA_CAPTURE_ERROR;
	}

	//取得したフレームデータのチェック
	if((Frame.cols <= 0) || (Frame.rows <= 0)){
		std::cout << MES_CAMERA_CAPTURE_DAMAGE << " = [" << uWIndex << "], " << "[" << Frame.cols << "], " << "[" << Frame.rows << "]" << std::endl;
		goto CAMERA_CAPTURE_ERROR;
	}

	//カメライメージ型の変数へ代入
	sWord_Type sWlength = ((Frame.channels() * Frame.cols) * Frame.rows);			//画像データの総サイズを計算（色データのサイズ×フレームの幅×フレームの高さ）
	m_MultiCaptureimage.images[uWIndex].width = Frame.cols;							//映像フレームの幅
	m_MultiCaptureimage.images[uWIndex].height = Frame.rows;						//映像フレームの高さ
	m_MultiCaptureimage.images[uWIndex].pixels.length(sWlength);					//データ長
	::memcpy((void *)&(m_MultiCaptureimage.images[uWIndex].pixels[0]), Frame.data, sWlength);	//データのコピー

	if(uWMatrixMode == CAPTURE_ONCE){
		//キャプチャ画像をJPGファイルに保存する
		uWResult = CameraCapJPGFileCreate(uWIndex);
		if(uWResult == RESULT_NG){
			uHWErrorLine = __LINE__;
			goto CAMERA_CAPTURE_ERROR;
		}
	}

	return RESULT_OK;

CAMERA_CAPTURE_ERROR:
#ifdef _DEBUG
	if(uHWErrorLine != 0){
		std::cout << "[ERROR MESSEGE]CameraCaptureByOpcv()Error = " << uHWErrorLine << std::endl;
	}
#endif

	return (RESULT_NG);
}

/*******************************************************************************
 *	Function Name		：MultiCameraSerialSort
 *	Description			：コンフィギュレーションで設定したシリアルNo順に
 * 						　キャプチャ画像の配列をソートする処理
 *	Argument			：In：カメラID
 *						：Out：ソート後のIndex
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::MultiCameraSerialSort(uWord_Type uWCameraID, uWord_Type *uWIndex_p)
{
	std::string ConfigSerialNoTbl[NM33_CONNECT_ENABLE_MAX] = {
		m_SerialNo_A,
		m_SerialNo_B,
		m_SerialNo_C,
		m_SerialNo_D,
		m_SerialNo_E,
		m_SerialNo_F,
		m_SerialNo_G,
		m_SerialNo_H,
	};

	uWord_Type uWSSKFunkResult = SSK_RESULT_SUCCESS;
	Char_Type * cSerialNo_p = "NM33";
	uWord_Type uWLoop = 0;
	uWord_Type uWIndex = NM33_CONNECT_ENABLE_MAX;

	if((uWCameraID > CAMERA_ID_MAX)){
		std::cout << MES_CAMERA_CONNECT_RANGE_OUT << std::endl;
		return (RESULT_NG);
	}

	if((uWIndex_p == NULL) || (m_NM33CameraSSKFunc_p == NULL)){
		return (RESULT_NG);
	}

	//カメラシリアルNo取得
	cSerialNo_p = m_NM33CameraSSKFunc_p->NM33CameraGetSerialNo((uWCameraID + 1));
	if(cSerialNo_p == NULL){
		std::cout << MES_SERIALNO_GET_ERROR << std::endl;
		return (RESULT_NG);
	}

	//カメラ接続最大数分コンフィギュレーションとドライバから取得したシリアルNoを比較する
	for(uWLoop = 0; uWLoop < NM33_CONNECT_ENABLE_MAX; uWLoop++){
		if(cSerialNo_p == ConfigSerialNoTbl[uWLoop]){
			uWIndex = uWLoop;
			break;
		}
	}
	
	if(uWIndex >= NM33_CONNECT_ENABLE_MAX){
		//コンフィギュレーションが空文字のケースや、接続対象外のカメラを使用している場合など
		std::cout << MES_CAMERA_EXEMPT_ERROR << std::endl;
		return (RESULT_NG);
	}else{
		*uWIndex_p = uWIndex;
	}

	return RESULT_OK;
}

/*******************************************************************************
 *	Function Name		：MultiCameraOpen
 *	Description			：OpenCVを使用したカメラオープン処理
 *	Argument			：なし
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::MultiCameraOpen(void)
{
	uWord_Type uWReturn = RESULT_OK;
	uWord_Type uWLoop = CAMERA_ID_0;

	if(m_uWBackUpCameraNum > NM33_CONNECT_ENABLE_MAX){
		return (RESULT_NG);
	}

	//カメラ接続台数分ビデオキャプチャをオープンする
	for(uWLoop = CAMERA_ID_0; uWLoop < m_uWBackUpCameraNum; uWLoop++){
		*VideoCapOpenTbl[uWLoop] = cv::VideoCapture(uWLoop);
		Bool_Type bRet = VideoCapOpenTbl[uWLoop]->isOpened();
		if(bRet == FALSE){
			std::cout << MES_CAMERA_OPEN_ERROR << uWLoop << std::endl;
			uWReturn = RESULT_NG;
			break;
		}
		VideoCapOpenTbl[uWLoop]->set(CV_CAP_PROP_FRAME_WIDTH, CAPTURE_FRAME_WIDTH);
		VideoCapOpenTbl[uWLoop]->set(CV_CAP_PROP_FRAME_HEIGHT, CAPTURE_FRAME_HEIGHT);
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：MultiCameraClose
 *	Description			：OpenCVを使用したカメラクローズ処理
 *	Argument			：なし
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::MultiCameraClose(void)
{
	uWord_Type uWReturn = RESULT_OK;
	uWord_Type uWLoop = CAMERA_ID_0;

	if(m_uWBackUpCameraNum > NM33_CONNECT_ENABLE_MAX){
		return (RESULT_NG);
	}

	//カメラ接続台数分ビデオキャプチャをクローズする
	for(uWLoop = CAMERA_ID_0; uWLoop < m_uWBackUpCameraNum; uWLoop++){
		VideoCapOpenTbl[uWLoop]->release();
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：CameraCapFileName
 *	Description			：カメラキャプチャ画像のファイル名作成処理
 *	Argument			：なし
 *	Return				：なし
 *******************************************************************************/
void NM33CameraCapture::CameraCapFileName(void)
{
	//キャプチャ開始前の時間を保存
	time_t stNow = time(NULL);
	struct tm *stNow_p = localtime(&stNow);

	//前回ファイル名を作成した時間と今回の時間を比較
	uWord_Type uWRet = ::memcmp(&m_stBackUpFileCreateTime, stNow_p, sizeof(tm));

	if(uWRet == 0){
		//一致していれば枝番をインクリメント
		m_uWFileBranchNum ++;
	}else{
		m_uWFileBranchNum = 0;
	}

	::memcpy(&m_stBackUpFileCreateTime, stNow_p, sizeof(tm));
}

/*******************************************************************************
 *	Function Name		：CameraCapJPGFileCreate
 *	Description			：JPGファイル生成処理
 *	Argument			：In：カメラID
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraCapture::CameraCapJPGFileCreate(uWord_Type uWIndex)
{
	const char *FileKindTbl[NM33_CONNECT_ENABLE_MAX] = {
		"A",
		"B",
		"C",
		"D",
		"E",
		"F",
		"G",
		"H",
	};

	uWord_Type uWReturn = RESULT_OK;

	if((uWIndex > CAMERA_ID_MAX)){
		std::cout << MES_CAMERA_CONNECT_RANGE_OUT << std::endl;
		return (RESULT_NG);
	}

	//格納フォルダの有無確認
	DWORD dWResult = GetFileAttributes("CaptureOutPut");

	if(dWResult == -1){
		//無ければ格納フォルダを作成
		BOOL bRet = CreateDirectory("CaptureOutPut", NULL);
		if(bRet == FALSE){
			//フォルダの生成に失敗
			std::cout << MES_FOLDER_CREATE_ERROR << std::endl;
			return (RESULT_NG);
		}
	}

	//静止画の場合JPGファイルの作成も行う
	Char_Type cFilename[128] = "";
	Char_Type cFileKindName[5] = "";
	
	//ファイルを識別するアルファベットを取得
	strcpy(cFileKindName, FileKindTbl[uWIndex]);

	//JPGファイルのファイル名称を生成
	::sprintf(cFilename, "CaptureOutPut/Capture%s_%04d%02d%02d%02d%02d%02d_%d.jpg", cFileKindName, (m_stBackUpFileCreateTime.tm_year + 1900), (m_stBackUpFileCreateTime.tm_mon + 1), 
			 m_stBackUpFileCreateTime.tm_mday, m_stBackUpFileCreateTime.tm_hour, m_stBackUpFileCreateTime.tm_min, m_stBackUpFileCreateTime.tm_sec, m_uWFileBranchNum);

	//対象カメラIDのキャプチャ画像をMAT型変数へコピー
	cv::Mat Frame = cv::Mat(cv::Size(m_MultiCaptureimage.images[uWIndex].width, m_MultiCaptureimage.images[uWIndex].height), CV_8UC3);
	::memcpy(Frame.data, (void *)&(m_MultiCaptureimage.images[uWIndex].pixels[0]), m_MultiCaptureimage.images[uWIndex].pixels.length());

	//JPGファイルへ書き込む
	BOOL bRet2 = imwrite(cFilename, Frame);
	if(bRet2 == FALSE){
		//ファイル書き込みに失敗
		std::cout << MES_CAMERA_CAP_CREATE_ERROR << std::endl;
		return (RESULT_NG);
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：CameraCapFPSLogOut
 *	Description			：キャプチャ周期のFPSログ出力
 *	Argument			：なし
 *	Return				：なし
 *******************************************************************************/
void NM33CameraCapture::CameraCapFPSLogOut(void)
{
	static coil::TimeValue LastTime = coil::gettimeofday();
	static uByte_Type uBCount = 0;

	if(uBCount > 100){
		uBCount = 0;
		coil::TimeValue Current_Time;
		Current_Time = coil::gettimeofday();
		double sec(Current_Time - LastTime);

		if (sec > 1.0 && sec < 1000.0){
			std::cout << "[MESSEGE]" << (100 / sec) << "FPS" << std::endl;
		}

		LastTime = Current_Time;
	}
	uBCount++;
}


extern "C"
{
 
  void NM33CameraCaptureInit(RTC::Manager* manager)
  {
    coil::Properties profile(nm33cameracapture_spec);
    manager->registerFactory(profile,
                             RTC::Create<NM33CameraCapture>,
                             RTC::Delete<NM33CameraCapture>);
  }
  
};


