/*******************************************************************************
 * File Name		: NM33CameraSSKFunc.cpp
 * Function			: NM33用のライブラリを使用しカメラ制御を行う
 * Author			: takashi sato
 * Date				: 2015/11/26
 * Last-modified by	: 
 * Last-modified	: 
 * History log		: 
 *					: 
 *******************************************************************************/

//Include宣言
#include "NM33CameraSSKFunc.h"


//コンストラクタ
NM33CameraSSKFunc::NM33CameraSSKFunc()
{
#ifdef _DEBUG
	std::cout << "[debug]NM33CameraSSKFunc()" << std::endl;
#endif
}

//デストラクタ
NM33CameraSSKFunc::~NM33CameraSSKFunc()
{
#ifdef _DEBUG
	std::cout << "[debug]~NM33CameraSSKFunc()" << std::endl;
#endif
}

/*******************************************************************************
 *	Function Name		：NM33CameraInit
 *	Description			：NM33カメラ接続の初期化を行う
 *	Argument			：なし
 *	Return				：正常終了/接続エラー/初期化エラー
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraInit(void)
{
	return NM33CameraInitWrap();
}

/*******************************************************************************
 *	Function Name		：NM33CameraDisconnect
 *	Description			：NM33カメラ接続を切断する
 *	Argument			：なし
 *	Return				：なし
 *******************************************************************************/
void	NM33CameraSSKFunc::NM33CameraDisconnect(void)
{
	NM33CameraDisconnectWrap();
}

/*******************************************************************************
 *	Function Name		：NM33CameraGetNumber
 *	Description			：NM33カメラの接続台数を取得する
 *	Argument			：なし
 *	Return				：カメラ接続台数
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraGetNumber(void)
{
	uWord_Type uWCameraNum = NM33_CONNECT_0;
	sWord_Type	sWResult = SSK_RESULT_SUCCESS;

	sWResult = NM33CameraGetNumbertWrap(&uWCameraNum);
	if(sWResult == SSK_RESULT_ERR){
		uWCameraNum = NM33_CONNECT_0;
	}

	return (uWCameraNum);
}

/*******************************************************************************
 *	Function Name		：NM33CameraGetSerialNo
 *	Description			：NM33カメラのシリアルNoを取得する
 *	Argument			：なし
 *	Return				：カメラ接続台数
 *******************************************************************************/
Char_Type*	NM33CameraSSKFunc::NM33CameraGetSerialNo(uWord_Type uWCameraNo)
{
	return NM33CameraGetSerialNoWrap(uWCameraNo);
}

/*******************************************************************************
 *	Function Name		：NM33CameraSetPanoramaMode
 *	Description			：NM33カメラの画面モードを設定する
 *	Argument			：なし
 *	Return				：カメラ接続台数
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraSetPanoramaMode(uWord_Type uWCameraNo, uWord_Type uWPanoramaMode)
{
	uWord_Type	uWPanoramaChgMode = PANORAMA_MODE_CIRCLE;

	switch(uWPanoramaMode){
		case	CHG_MODE_DEFAULT:
			uWPanoramaChgMode = PANORAMA_MODE_CIRCLE;
			break;
		case	CHG_MODE_PERSPECTIVE:
		case	CHG_MODE_BIRDS_EYE:
			uWPanoramaChgMode = PANORAMA_MODE_PERSPECTIVE;
			break;
		default:
			break;
	}

	return NM33CameraSetPanoramaModeWrap(uWCameraNo, uWPanoramaChgMode);
}

/*==============================================================================
					NM33 SSK用のWrapper関数	
==============================================================================*/
/*******************************************************************************
 *	Function Name		：NM33CameraInitWrap
 *	Description			：カメラの接続を初期化する
 *	Argument			：なし
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type NM33CameraSSKFunc::NM33CameraInitWrap(void)
{
	sWord_Type	sWResult = 0;
	uWord_Type	uWReturn = SSK_RESULT_SUCCESS;

	sWResult = nm30_init();

	switch (sWResult){
		case	0:
		//正常終了
			uWReturn = SSK_RESULT_SUCCESS;
			break;
		case	1:
		//接続エラー
			uWReturn = SSK_RESULT_CONNECT_ERR;
			break;
		case	2:
		default:
		//初期化エラー
			uWReturn = SSK_RESULT_INIT_ERR;
			break;
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：NM33CameraDisconnectWrap
 *	Description			：カメラの接続を切断する
 *	Argument			：なし
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraDisconnectWrap(void)
{
	sWord_Type	sWResult = 0;
	uWord_Type	uWReturn = SSK_RESULT_SUCCESS;

	sWResult = nm30_disconnect();
	if(sWResult == 0){
		uWReturn = SSK_RESULT_SUCCESS;
	}else{
		//nm30_disconnectの戻り値は0のみとなっているが念の為
		uWReturn = SSK_RESULT_ERR;
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：NM33CameraGetNumbertWrap
 *	Description			：接続されているカメラの数を取得する
 *	Argument			：out：カメラ接続台数を格納するポインタ
 *							  ※戻り値が正常終了の時に値が入る
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraGetNumbertWrap(uWord_Type *uWCameraNum_p)
{
	sWord_Type	sWResult = 0;
	uWord_Type	uWReturn = SSK_RESULT_SUCCESS;
	uWord_Type	uWRetryCnt = 0;

	if(uWCameraNum_p == NULL){
		return (SSK_RESULT_ERR);
	}

	for(uWRetryCnt = 0; uWRetryCnt <= SSK_ERROR_RETRY_MAX; uWRetryCnt++){
		sWResult = nm30_get_number();
		if(sWResult == 0){
			//不定値の為Retryする
			std::cout << "[debug]SSK Error nm30_get_number() = [" << uWRetryCnt << "]RetryCnt" << std::endl;
			Sleep(100);
		}else{
			break;
		}
	}

	if(sWResult == 0){
		//0は不定値になるため、エラーを返す
		return (SSK_RESULT_ERR);
	}

	if(sWResult > 8){
		*uWCameraNum_p = NM33_CONNECT_OVER;
	}else{
		*uWCameraNum_p = sWResult;
	}

	return (uWReturn);
}


/*******************************************************************************
 *	Function Name		：NM33CameraSetPanoramaModeWrap
 *	Description			：画面モードを設定する
 *	Argument			：in：カメラID(1～8)/画面モード
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraSetPanoramaModeWrap(uWord_Type uWCameraNo, uWord_Type uWPanoramaChgMode)
{
	sWord_Type	sWResult = 0;
	uWord_Type	uWReturn = SSK_RESULT_SUCCESS;
	uWord_Type	uWRetryCnt = 0;

	if((uWCameraNo > NM33_CONNECT_ENABLE_MAX) || (uWCameraNo == 0)){
		//8台以上のカメラを接続している場合はエラーを返す
		return (SSK_RESULT_ERR);
	}

	if((uWPanoramaChgMode >= PANORAMA_MODE_MAX) ||
		(uWPanoramaChgMode == PANORAMA_MODE_DUMMY_1) || (uWPanoramaChgMode == PANORAMA_MODE_DUMMY_2) || (uWPanoramaChgMode == PANORAMA_MODE_DUMMY_3)){
		//設定する画面モードのパラメータが範囲外の場合はエラーを返す
		return ( SSK_RESULT_ERR );
	}

	for(uWRetryCnt = 0; uWRetryCnt <= SSK_ERROR_RETRY_MAX; uWRetryCnt++){
		sWResult = nm30_set_panorama_mode(static_cast<sWord_Type>(uWCameraNo), static_cast<sWord_Type>(uWPanoramaChgMode));
		if(sWResult == 0){
			uWReturn = SSK_RESULT_SUCCESS;
			break;
		}else{
			//0以外は異常終了の為Retryする
			std::cout << "[debug]SSK Error nm30_set_panorama_mode() = [" << uWRetryCnt << "]RetryCnt" << std::endl;
			uWReturn = SSK_RESULT_ERR;
			Sleep(100);
		}
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：NM33CameraGetPanoramaModeWrap
 *	Description			：現在の画面モードを取得する
 *	Argument			：in：カメラID(1～8)
 *						：Out：画面モードを格納するポインタ
 *	Return				：正常終了/異常終了
 *******************************************************************************/
uWord_Type	NM33CameraSSKFunc::NM33CameraGetPanoramaModeWrap(uWord_Type uWCameraNo, uWord_Type *uWPanoramaMode_p)
{
	sWord_Type	sWResult = 0;
	uWord_Type	uWReturn = SSK_RESULT_SUCCESS;

	if((uWCameraNo > NM33_CONNECT_ENABLE_MAX) || (uWCameraNo == 0)){
		//8台以上のカメラを接続している場合はエラーを返す
		return (SSK_RESULT_ERR);
	}

	if(uWPanoramaMode_p == NULL){
		return (SSK_RESULT_ERR);
	}

	sWResult = nm30_get_panorama_mode (static_cast<sWord_Type>(uWCameraNo));

	switch(sWResult){
		case	0:
			//円形
			*uWPanoramaMode_p = PANORAMA_MODE_CIRCLE;
			break;
		case	1:
			//周回
			*uWPanoramaMode_p = PANORAMA_MODE_AROUND;
			break;
		case	2:
			//周回（反転）
			*uWPanoramaMode_p = PANORAMA_MODE_AROUND_TURN;
			break;
		case	3:
			//上下二分割
			*uWPanoramaMode_p = PANORAMA_MODE_HALF;
			break;
		case	4:
			//上下二分割（反転）
			*uWPanoramaMode_p = PANORAMA_MODE_HALF_TURN;
			break;
		case	5:
			//4画面
			*uWPanoramaMode_p = PANORAMA_MODE_FOUR_SCREEN;
			break;		
		case	6:
			//4画面（反転）
			*uWPanoramaMode_p = PANORAMA_MODE_FOUR_SCREEN_TURN;
			break;
		case	9:
			//円形（Largeサイズ）
			*uWPanoramaMode_p = PANORAMA_MODE_CIRCLE_LARGE;
			break;
		case	11:
			//直交一画面
			*uWPanoramaMode_p = PANORAMA_MODE_PERSPECTIVE;
			break;
		default:
			uWReturn = SSK_RESULT_ERR;
			break;
	}

	return (uWReturn);
}

/*******************************************************************************
 *	Function Name		：NM33CameraGetPanoramaModeWrap
 *	Description			：映像フレームの解像度を取得する
 *	Argument			：in：カメラID(1～8)
 *	Return				：正常終了：シリアルNoの文字列を返す/異常終了：NULLを返す
 *******************************************************************************/
Char_Type*	NM33CameraSSKFunc::NM33CameraGetSerialNoWrap(uWord_Type uWCameraNo)
{
	Char_Type*	cHSerialNo = "NM33";
	uWord_Type	uWRetryCnt = 0;

	if((uWCameraNo > NM33_CONNECT_ENABLE_MAX) || (uWCameraNo == 0)){
		//8台以上のカメラを接続している場合はエラーを返す
		return (NULL);
	}

	for(uWRetryCnt = 0; uWRetryCnt <= SSK_ERROR_RETRY_MAX; uWRetryCnt++){
		cHSerialNo = nm30_get_id(uWCameraNo);
		if(cHSerialNo == NULL){
			//NULLの場合、異常終了と判断する。
			std::cout << "[debug]SSK Error nm30_get_id() = [" << uWRetryCnt << "]RetryCnt" << std::endl;
			Sleep(100);
		}else{
			break;
		}
	}

	return (cHSerialNo);
}

